"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

function isObject(obj) {
  return obj !== null && typeof obj === "object" && !Array.isArray(obj);
}

function uppercaseFirstChar(str) {
  return str.charAt(0).toUpperCase() + str.slice(1);
}

var generateFragmentFromPropsFunctionName = "generateFragmentFromProps";

exports["default"] = function (_ref) {
  var _this = this,
      _arguments = arguments;

  var Plugin = _ref.Plugin;
  var t = _ref.types;

  // t.templateElement does not pass args
  // https://github.com/babel/babel/blob/master/packages/babel/src/types/definitions/es2015.js#L123
  var originalTemplateElement = t.templateElement;
  t.templateElement = function (value) {
    var tail = arguments.length <= 1 || arguments[1] === undefined ? true : arguments[1];

    var templateElement = originalTemplateElement.apply(_this, _arguments);
    return _extends({}, templateElement, { value: value, tail: tail });
  };

  function flowTypeAnnotationToString(type) {
    if (t.isStringTypeAnnotation(type)) {
      return "string";
    } else if (t.isNumberTypeAnnotation(type)) {
      return "number";
    } else if (t.isBooleanTypeAnnotation(type)) {
      return "boolean";
    }
    return "any";
  }

  // convert an ObjectTypeAnnotation to a js object
  function convertFlowObjectTypeAnnotation(objectType) {
    return objectType.properties.reduce(function (obj, property) {
      var key = property.key.name;
      var value = property.value;
      if (t.isObjectTypeAnnotation(value)) {
        obj[key] = convertFlowObjectTypeAnnotation(value);
      } else {
        obj[key] = flowTypeAnnotationToString(value);
      }
      return obj;
    }, {});
  }

  function convertToGraphQLString(obj) {
    var level = arguments.length <= 1 || arguments[1] === undefined ? 1 : arguments[1];

    var indentation = "  ".repeat(level);

    var strings = Object.keys(obj).reduce(function (parts, key) {
      var value = obj[key];
      var str = key;
      if (isObject(value)) {
        str = str + " {\n" + convertToGraphQLString(value, level + 1) + "\n" + indentation + "}";
      }
      parts.push(indentation + str);
      return parts;
    }, []);

    return strings.join(",\n");
  }

  function storeTypeAlias(node, state) {
    state.opts.extra.flowTypes = state.opts.extra.flowTypes || {};
    state.opts.extra.flowTypes[node.id.name] = node;
  }

  function storeClassPropTypes(state, className, propType) {
    state.opts.extra.classPropTypes = state.opts.extra.classPropTypes || {};
    state.opts.extra.classPropTypes[className] = propType;
  }

  return new Plugin("flow-relay-query", {
    visitor: {
      TypeAlias: function TypeAlias(node, parent, scope, state) {
        storeTypeAlias(node, state);
      },

      ClassProperty: function ClassProperty(node, parent, scope, state) {
        if (node.key.name !== "props") {
          return;
        }

        var typeAnnotation = node.typeAnnotation && node.typeAnnotation.typeAnnotation;
        if (!typeAnnotation || !t.isGenericTypeAnnotation(typeAnnotation)) {
          return;
        }

        var classDec = this.findParent(function (p) {
          return t.isClassDeclaration(p);
        });
        var superClass = classDec && classDec.node && classDec.node.superClass;
        if (!superClass || superClass.object.name !== "React" || superClass.property.name !== "Component") {
          return;
        }

        var className = classDec.node.id.name;
        var propType = typeAnnotation.id.name;

        storeClassPropTypes(state, className, propType);
      },

      ImportDeclaration: function ImportDeclaration(node, parent, scope, state) {
        // handle imported types
        if (node.importKind === "type" || node.importKind === "typeof") {
          (function () {
            var typeNames = node.specifiers.map(function (specifier) {
              return specifier.local.name;
            });
            var filename = state.opts.filename;

            if (!filename) {
              throw new Error("Imported types " + typeNames.join(", ") + " but there is no current file");
            }

            var importFile = node.source.value;
            var importFilename = _path2["default"].resolve(_path2["default"].dirname(filename), importFile + ".js");

            // hack to get the imported type
            var babel = require("babel-core");
            var program = babel.parse(_fs2["default"].readFileSync(importFilename, "utf8"));

            var visitor = {
              TypeAlias: function TypeAlias(n) {
                var typeName = n.id.name;
                if (typeNames.indexOf(typeName) >= 0) {
                  storeTypeAlias(n, state);
                }
              }
            };
            babel.traverse(program, visitor, null, program);
          })();
        }

        // remove the marker function import
        var remove = node.specifiers.filter(function (specifier) {
          return specifier.local.name === generateFragmentFromPropsFunctionName;
        }).length > 0;

        if (remove) {
          this.dangerouslyRemove();
        }
      },

      CallExpression: function CallExpression(node, parent, scope, state) {
        if (!t.isIdentifier(node.callee, { name: generateFragmentFromPropsFunctionName })) {
          return this;
        }

        var relayCreateContainer = this.findParent(function (p) {
          return t.isCallExpression(p) && p.get("callee").matchesPattern("Relay.createContainer");
        });

        if (!relayCreateContainer) {
          return this;
        }

        var reactClassName = relayCreateContainer.node.arguments[0].name;
        var typeName = state.opts.extra.classPropTypes && state.opts.extra.classPropTypes[reactClassName];
        if (!typeName) {
          throw new Error("React class " + reactClassName + " does not have flow typed props");
        }

        var type = state.opts.extra.flowTypes && state.opts.extra.flowTypes[typeName];
        if (!type) {
          throw new Error("There is no flow type with name " + typeName);
        }

        var typeKey = t.isProperty(parent) && parent.key.name;
        var typeProperties = type.right.properties;

        var typeAtKey = null;
        for (var i = 0; i < typeProperties.length; i++) {
          var typeProperty = typeProperties[i];
          if (t.isObjectTypeProperty(typeProperty) && typeProperty.key.name === typeKey) {
            typeAtKey = typeProperty.value;
          }
        }

        if (!typeAtKey) {
          throw new Error("There is no property named '" + typeKey + "' in flow type " + typeName);
        }

        if (!t.isObjectTypeAnnotation(typeAtKey)) {
          throw new Error("Flow type " + typeName + "." + typeKey + " is not an object");
        }

        var obj = convertFlowObjectTypeAnnotation(typeAtKey);
        var graphQlQueryBody = convertToGraphQLString(obj);

        var fragmentName = node.arguments[1] && node.arguments[1].value || uppercaseFirstChar(typeKey);
        var graphQlQuery = "\n" + "fragment on " + fragmentName + " {\n" + graphQlQueryBody + "\n" + "}\n";

        var relayQl = t.taggedTemplateExpression(t.memberExpression(t.identifier("Relay"), t.identifier("QL")), t.templateLiteral([t.templateElement({ cooked: graphQlQuery, raw: graphQlQuery })], []));
        return t.arrowFunctionExpression([], relayQl);
      }
    }
  });
};

module.exports = exports["default"];